function Jt=RiseJacobian(q11,q12,q21,q22,q31,q32)

J1=RiseLegJacobian('Right',q11,q12);
J2=RiseLegJacobian('Left',q21,q22);
J3=RiseLegJacobian('Right',q31,q32);

Jt=[J1 zeros(3,4); zeros(3,2) J2 zeros(3,2); zeros(3,4) J3];
